@@ -1,8 +1,9 @@
-import distutils
+import hashlib
 import os
 import re
 import ssl
 import sys
+import sysconfig
 import textwrap
 from os.path import curdir, join, pardir
 from pathlib import Path
@@ -13,6 +14,7 @@ import pytest
 from pip._internal.cli.status_codes import ERROR, SUCCESS
 from pip._internal.models.index import PyPI, TestPyPI
 from pip._internal.utils.misc import rmtree
+from pip._internal.utils.urls import path_to_url
 from tests.conftest import CertFactory
 from tests.lib import (
     PipTestEnvironment,
@@ -170,7 +172,7 @@ def test_pep518_allows_missing_requires(
     assert result.files_created
 
 
-@pytest.mark.incompatible_with_test_venv
+@pytest.mark.usefixtures("enable_user_site")
 def test_pep518_with_user_pip(
     script: PipTestEnvironment, pip_src: Path, data: TestData, common_wheels: Path
 ) -> None:
@@ -183,7 +185,16 @@ def test_pep518_with_user_pip(
     non-isolated environment, and break pip in the system site-packages,
     so that isolated uses of pip will fail.
     """
-    script.pip("install", "--ignore-installed", "-f", common_wheels, "--user", pip_src)
+    script.pip(
+        "install",
+        "--ignore-installed",
+        "-f",
+        common_wheels,
+        "--user",
+        pip_src,
+        # WARNING: The scripts pip, pip3, ... are installed in ... which is not on PATH
+        allow_stderr_warning=True,
+    )
     system_pip_dir = script.site_packages_path / "pip"
     assert not system_pip_dir.exists()
     system_pip_dir.mkdir()
@@ -263,7 +274,6 @@ def test_pep518_forkbombs(
 
 
 @pytest.mark.network
-@pytest.mark.usefixtures("with_wheel")
 def test_pip_second_command_line_interface_works(
     script: PipTestEnvironment,
     pip_src: Path,
@@ -308,7 +318,6 @@ def test_install_exit_status_code_when_blank_requirements_file(
 
 
 @pytest.mark.network
-@pytest.mark.usefixtures("with_wheel")
 def test_basic_install_from_pypi(script: PipTestEnvironment) -> None:
     """
     Test installing a package from PyPI.
@@ -367,7 +376,6 @@ def test_basic_install_editable_from_git(script: PipTestEnvironment) -> None:
     _test_install_editable_from_git(script)
 
 
-@pytest.mark.usefixtures("with_wheel")
 def test_install_editable_from_git_autobuild_wheel(script: PipTestEnvironment) -> None:
     _test_install_editable_from_git(script)
 
@@ -494,7 +502,6 @@ def test_vcs_url_urlquote_normalization(
 
 
 @pytest.mark.parametrize("resolver", ["", "--use-deprecated=legacy-resolver"])
-@pytest.mark.usefixtures("with_wheel")
 def test_basic_install_from_local_directory(
     script: PipTestEnvironment, data: TestData, resolver: str
 ) -> None:
@@ -524,7 +531,6 @@ def test_basic_install_from_local_directory(
         ("embedded_rel_path", True),
     ],
 )
-@pytest.mark.usefixtures("with_wheel")
 def test_basic_install_relative_directory(
     script: PipTestEnvironment, data: TestData, test_type: str, editable: bool
 ) -> None:
@@ -612,6 +618,159 @@ def test_hashed_install_failure(script: PipTestEnvironment, tmpdir: Path) -> Non
     assert len(result.files_created) == 0
 
 
+def test_link_hash_pass_require_hashes(
+    script: PipTestEnvironment, shared_data: TestData
+) -> None:
+    """Test that a good hash in user provided direct URL is
+    considered valid for --require-hashes."""
+    url = path_to_url(str(shared_data.packages.joinpath("simple-1.0.tar.gz")))
+    url = (
+        f"{url}#sha256="
+        "393043e672415891885c9a2a0929b1af95fb866d6ca016b42d2e6ce53619b653"
+    )
+    script.pip_install_local("--no-deps", "--require-hashes", url)
+
+
+def test_bad_link_hash_install_failure(
+    script: PipTestEnvironment, shared_data: TestData
+) -> None:
+    """Test that wrong hash in direct URL stops installation."""
+    url = path_to_url(str(shared_data.packages.joinpath("simple-1.0.tar.gz")))
+    url = f"{url}#sha256=invalidhash"
+    result = script.pip_install_local("--no-deps", url, expect_error=True)
+    assert "THESE PACKAGES DO NOT MATCH THE HASHES" in result.stderr
+
+
+def test_bad_link_hash_good_user_hash_install_success(
+    script: PipTestEnvironment, shared_data: TestData, tmp_path: Path
+) -> None:
+    """Test that wrong hash in direct URL ignored when good --hash provided.
+
+    This behaviour may be accidental?
+    """
+    url = path_to_url(str(shared_data.packages.joinpath("simple-1.0.tar.gz")))
+    url = f"{url}#sha256=invalidhash"
+    digest = "393043e672415891885c9a2a0929b1af95fb866d6ca016b42d2e6ce53619b653"
+    with requirements_file(
+        f"simple @ {url} --hash sha256:{digest}", tmp_path
+    ) as reqs_file:
+        script.pip_install_local("--no-deps", "--require-hashes", "-r", reqs_file)
+
+
+def test_link_hash_in_dep_fails_require_hashes(
+    script: PipTestEnvironment, tmp_path: Path, shared_data: TestData
+) -> None:
+    """Test that a good hash in direct URL dependency is not considered
+    for --require-hashes."""
+    # Create a project named pkga that depends on the simple-1.0.tar.gz with a direct
+    # URL including a hash.
+    simple_url = path_to_url(str(shared_data.packages.joinpath("simple-1.0.tar.gz")))
+    simple_url_with_hash = (
+        f"{simple_url}#sha256="
+        "393043e672415891885c9a2a0929b1af95fb866d6ca016b42d2e6ce53619b653"
+    )
+    project_path = tmp_path / "pkga"
+    project_path.mkdir()
+    project_path.joinpath("pyproject.toml").write_text(
+        textwrap.dedent(
+            f"""\
+            [project]
+            name = "pkga"
+            version = "1.0"
+            dependencies = ["simple @ {simple_url_with_hash}"]
+            """
+        )
+    )
+    # Build a wheel for pkga and compute its hash.
+    wheelhouse = tmp_path / "wheehouse"
+    wheelhouse.mkdir()
+    script.pip("wheel", "--no-deps", "-w", wheelhouse, project_path)
+    digest = hashlib.sha256(
+        wheelhouse.joinpath("pkga-1.0-py3-none-any.whl").read_bytes()
+    ).hexdigest()
+    # Install pkga from a requirements file with hash, using --require-hashes.
+    # This should fail because we have not provided a hash for the 'simple' dependency.
+    with requirements_file(f"pkga==1.0 --hash sha256:{digest}", tmp_path) as reqs_file:
+        result = script.pip(
+            "install",
+            "--no-build-isolation",
+            "--require-hashes",
+            "--no-index",
+            "-f",
+            wheelhouse,
+            "-r",
+            reqs_file,
+            expect_error=True,
+        )
+    assert "Hashes are required in --require-hashes mode" in result.stderr
+
+
+def test_bad_link_hash_in_dep_install_failure(
+    script: PipTestEnvironment, tmp_path: Path, shared_data: TestData
+) -> None:
+    """Test that wrong hash in direct URL dependency stops installation."""
+    url = path_to_url(str(shared_data.packages.joinpath("simple-1.0.tar.gz")))
+    url = f"{url}#sha256=invalidhash"
+    project_path = tmp_path / "pkga"
+    project_path.mkdir()
+    project_path.joinpath("pyproject.toml").write_text(
+        textwrap.dedent(
+            f"""\
+            [project]
+            name = "pkga"
+            version = "1.0"
+            dependencies = ["simple @ {url}"]
+            """
+        )
+    )
+    result = script.pip_install_local(
+        "--no-build-isolation", project_path, expect_error=True
+    )
+    assert "THESE PACKAGES DO NOT MATCH THE HASHES" in result.stderr, result.stderr
+
+
+def test_hashed_install_from_cache(
+    script: PipTestEnvironment, data: TestData, tmpdir: Path
+) -> None:
+    """
+    Test that installing from a cached built wheel works and that the hash is verified
+    against the hash of the original source archived stored in the cache entry.
+    """
+    with requirements_file(
+        "simple2==1.0 --hash=sha256:"
+        "9336af72ca661e6336eb87bc7de3e8844d853e3848c2b9bbd2e8bf01db88c2c7\n",
+        tmpdir,
+    ) as reqs_file:
+        result = script.pip_install_local(
+            "--use-pep517", "--no-build-isolation", "-r", reqs_file.resolve()
+        )
+        assert "Created wheel for simple2" in result.stdout
+        script.pip("uninstall", "simple2", "-y")
+        result = script.pip_install_local(
+            "--use-pep517", "--no-build-isolation", "-r", reqs_file.resolve()
+        )
+        assert "Using cached simple2" in result.stdout
+    # now try with an invalid hash
+    with requirements_file(
+        "simple2==1.0 --hash=sha256:invalid\n",
+        tmpdir,
+    ) as reqs_file:
+        script.pip("uninstall", "simple2", "-y")
+        result = script.pip_install_local(
+            "--use-pep517",
+            "--no-build-isolation",
+            "-r",
+            reqs_file.resolve(),
+            expect_error=True,
+        )
+        assert (
+            "WARNING: The hashes of the source archive found in cache entry "
+            "don't match, ignoring cached built wheel and re-downloading source."
+        ) in result.stderr
+        assert "Using cached simple2" in result.stdout
+        assert "ERROR: THESE PACKAGES DO NOT MATCH THE HASHES" in result.stderr
+
+
 def assert_re_match(pattern: str, text: str) -> None:
     assert re.search(pattern, text), f"Could not find {pattern!r} in {text!r}"
 
@@ -646,7 +805,6 @@ def test_hashed_install_failure_later_flag(
     )
 
 
-@pytest.mark.usefixtures("with_wheel")
 def test_install_from_local_directory_with_in_tree_build(
     script: PipTestEnvironment, data: TestData
 ) -> None:
@@ -690,17 +848,27 @@ def test_editable_install__local_dir_no_setup_py(
     )
 
 
+@pytest.mark.network
 def test_editable_install__local_dir_no_setup_py_with_pyproject(
     script: PipTestEnvironment,
 ) -> None:
     """
     Test installing in editable mode from a local directory with no setup.py
-    but that does have pyproject.toml.
+    but that does have pyproject.toml with a build backend that does not support
+    the build_editable hook.
     """
     local_dir = script.scratch_path.joinpath("temp")
     local_dir.mkdir()
     pyproject_path = local_dir.joinpath("pyproject.toml")
-    pyproject_path.write_text("")
+    pyproject_path.write_text(
+        textwrap.dedent(
+            """
+                [build-system]
+                requires = ["setuptools<64"]
+                build-backend = "setuptools.build_meta"
+            """
+        )
+    )
 
     result = script.pip("install", "-e", local_dir, expect_error=True)
     assert not result.files_created
@@ -781,7 +949,6 @@ def test_upgrade_argparse_shadowed(script: PipTestEnvironment) -> None:
     assert "Not uninstalling argparse" not in result.stdout
 
 
-@pytest.mark.usefixtures("with_wheel")
 def test_install_curdir(script: PipTestEnvironment, data: TestData) -> None:
     """
     Test installing current directory ('.').
@@ -798,7 +965,6 @@ def test_install_curdir(script: PipTestEnvironment, data: TestData) -> None:
     result.did_create(dist_info_folder)
 
 
-@pytest.mark.usefixtures("with_wheel")
 def test_install_pardir(script: PipTestEnvironment, data: TestData) -> None:
     """
     Test installing parent directory ('..').
@@ -818,10 +984,15 @@ def test_install_global_option(script: PipTestEnvironment) -> None:
     (In particular those that disable the actual install action)
     """
     result = script.pip(
-        "install", "--global-option=--version", "INITools==0.1", expect_stderr=True
+        "install",
+        "--global-option=--version",
+        "INITools==0.1",
+        expect_error=True,  # build is going to fail because of --version
     )
     assert "INITools==0.1\n" in result.stdout
     assert not result.files_created
+    assert "Implying --no-binary=:all:" in result.stderr
+    assert "A possible replacement is to use --config-settings" in result.stderr
 
 
 def test_install_with_hacked_egg_info(
@@ -835,29 +1006,6 @@ def test_install_with_hacked_egg_info(
     assert "Successfully installed hackedegginfo-0.0.0\n" in result.stdout
 
 
-@pytest.mark.network
-def test_install_using_install_option_and_editable(
-    script: PipTestEnvironment, tmpdir: Path
-) -> None:
-    """
-    Test installing a tool using -e and --install-option
-    """
-    folder = "script_folder"
-    script.scratch_path.joinpath(folder).mkdir()
-    url = local_checkout("git+https://github.com/pypa/pip-test-package", tmpdir)
-    result = script.pip(
-        "install",
-        "-e",
-        f"{url}#egg=pip-test-package",
-        f"--install-option=--script-dir={folder}",
-        expect_stderr=True,
-    )
-    script_file = (
-        script.venv / "src/pip-test-package" / folder / f"pip-test-package{script.exe}"
-    )
-    result.did_create(script_file)
-
-
 @pytest.mark.xfail
 @pytest.mark.network
 @need_mercurial
@@ -879,7 +1027,6 @@ def test_install_global_option_using_editable(
 
 
 @pytest.mark.network
-@pytest.mark.usefixtures("with_wheel")
 def test_install_package_with_same_name_in_curdir(script: PipTestEnvironment) -> None:
     """
     Test installing a package with the same name of a local folder
@@ -898,7 +1045,6 @@ mock100_setup_py = textwrap.dedent(
 )
 
 
-@pytest.mark.usefixtures("with_wheel")
 def test_install_folder_using_dot_slash(script: PipTestEnvironment) -> None:
     """
     Test installing a folder using pip install ./foldername
@@ -911,7 +1057,6 @@ def test_install_folder_using_dot_slash(script: PipTestEnvironment) -> None:
     result.did_create(dist_info_folder)
 
 
-@pytest.mark.usefixtures("with_wheel")
 def test_install_folder_using_slash_in_the_end(script: PipTestEnvironment) -> None:
     r"""
     Test installing a folder using pip install foldername/ or foldername\
@@ -924,7 +1069,6 @@ def test_install_folder_using_slash_in_the_end(script: PipTestEnvironment) -> No
     result.did_create(dist_info_folder)
 
 
-@pytest.mark.usefixtures("with_wheel")
 def test_install_folder_using_relative_path(script: PipTestEnvironment) -> None:
     """
     Test installing a folder using pip install folder1/folder2
@@ -939,7 +1083,6 @@ def test_install_folder_using_relative_path(script: PipTestEnvironment) -> None:
 
 
 @pytest.mark.network
-@pytest.mark.usefixtures("with_wheel")
 def test_install_package_which_contains_dev_in_name(script: PipTestEnvironment) -> None:
     """
     Test installing package from PyPI which contains 'dev' in name
@@ -951,7 +1094,6 @@ def test_install_package_which_contains_dev_in_name(script: PipTestEnvironment)
     result.did_create(dist_info_folder)
 
 
-@pytest.mark.usefixtures("with_wheel")
 def test_install_package_with_target(script: PipTestEnvironment) -> None:
     """
     Test installing a package using pip install --target
@@ -1084,7 +1226,6 @@ def test_install_nonlocal_compatible_wheel_path(
 
 
 @pytest.mark.parametrize("opt", ("--target", "--prefix"))
-@pytest.mark.usefixtures("with_wheel")
 def test_install_with_target_or_prefix_and_scripts_no_warning(
     opt: str, script: PipTestEnvironment
 ) -> None:
@@ -1123,7 +1264,6 @@ def test_install_with_target_or_prefix_and_scripts_no_warning(
     assert "--no-warn-script-location" not in result.stderr, str(result)
 
 
-@pytest.mark.usefixtures("with_wheel")
 def test_install_package_with_root(script: PipTestEnvironment, data: TestData) -> None:
     """
     Test installing a package using pip install --root
@@ -1141,8 +1281,9 @@ def test_install_package_with_root(script: PipTestEnvironment, data: TestData) -
     normal_install_path = os.fspath(
         script.base_path / script.site_packages / "simple-1.0.dist-info"
     )
-    # use distutils to change the root exactly how the --root option does it
-    from distutils.util import change_root
+    # use a function borrowed from distutils
+    # to change the root exactly how the --root option does it
+    from pip._internal.locations.base import change_root
 
     root_path = change_root(os.path.join(script.scratch, "root"), normal_install_path)
     result.did_create(root_path)
@@ -1173,9 +1314,8 @@ def test_install_package_with_prefix(
 
     rel_prefix_path = script.scratch / "prefix"
     install_path = join(
-        distutils.sysconfig.get_python_lib(prefix=rel_prefix_path),
-        # we still test for egg-info because no-binary implies setup.py install
-        f"simple-1.0-py{pyversion}.egg-info",
+        sysconfig.get_path("purelib", vars={"base": rel_prefix_path}),
+        "simple-1.0.dist-info",
     )
     result.did_create(install_path)
 
@@ -1195,7 +1335,7 @@ def _test_install_editable_with_prefix(
             "prefix", "lib", f"python{pyversion}", "site-packages"
         )
     else:
-        site_packages = distutils.sysconfig.get_python_lib(prefix="prefix")
+        site_packages = sysconfig.get_path("purelib", vars={"base": "prefix"})
 
     # make sure target path is in PYTHONPATH
     pythonpath = script.scratch_path / site_packages
@@ -1243,13 +1383,14 @@ setup(name='pkga', version='0.1')
     _test_install_editable_with_prefix(script, {"setup.py": setup_py})
 
 
+@pytest.mark.network
 def test_install_editable_with_prefix_setup_cfg(script: PipTestEnvironment) -> None:
     setup_cfg = """[metadata]
 name = pkga
 version = 0.1
 """
     pyproject_toml = """[build-system]
-requires = ["setuptools", "wheel"]
+requires = ["setuptools<64", "wheel"]
 build-backend = "setuptools.build_meta"
 """
     _test_install_editable_with_prefix(
@@ -1318,7 +1459,6 @@ def test_install_package_with_latin1_setup(
     script.pip("install", to_install)
 
 
-@pytest.mark.usefixtures("with_wheel")
 def test_url_req_case_mismatch_no_index(
     script: PipTestEnvironment, data: TestData
 ) -> None:
@@ -1342,7 +1482,6 @@ def test_url_req_case_mismatch_no_index(
     result.did_not_create(dist_info_folder)
 
 
-@pytest.mark.usefixtures("with_wheel")
 def test_url_req_case_mismatch_file_index(
     script: PipTestEnvironment, data: TestData
 ) -> None:
@@ -1372,7 +1511,6 @@ def test_url_req_case_mismatch_file_index(
     result.did_not_create(dist_info_folder)
 
 
-@pytest.mark.usefixtures("with_wheel")
 def test_url_incorrect_case_no_index(
     script: PipTestEnvironment, data: TestData
 ) -> None:
@@ -1396,7 +1534,6 @@ def test_url_incorrect_case_no_index(
     result.did_create(dist_info_folder)
 
 
-@pytest.mark.usefixtures("with_wheel")
 def test_url_incorrect_case_file_index(
     script: PipTestEnvironment, data: TestData
 ) -> None:
@@ -1515,12 +1652,15 @@ def test_install_subprocess_output_handling(
     # If the install fails, then we *should* show the output... but only once,
     # even if --verbose is given.
     result = script.pip(*(args + ["--global-option=--fail"]), expect_error=True)
-    assert 1 == result.stderr.count("I DIE, I DIE")
+    # This error is emitted 3 times:
+    # - by setup.py bdist_wheel
+    # - by setup.py clean
+    assert 2 == result.stderr.count("I DIE, I DIE")
 
     result = script.pip(
         *(args + ["--global-option=--fail", "--verbose"]), expect_error=True
     )
-    assert 1 == result.stderr.count("I DIE, I DIE")
+    assert 2 == result.stderr.count("I DIE, I DIE")
 
 
 def test_install_log(script: PipTestEnvironment, data: TestData, tmpdir: Path) -> None:
@@ -1540,27 +1680,14 @@ def test_install_topological_sort(script: PipTestEnvironment, data: TestData) ->
     assert order1 in res or order2 in res, res
 
 
-@pytest.mark.usefixtures("with_wheel")
-def test_install_wheel_broken(script: PipTestEnvironment) -> None:
-    res = script.pip_install_local("wheelbroken", expect_stderr=True)
-    assert "Successfully installed wheelbroken-0.1" in str(res), str(res)
-
-
-@pytest.mark.usefixtures("with_wheel")
 def test_cleanup_after_failed_wheel(script: PipTestEnvironment) -> None:
-    res = script.pip_install_local("wheelbrokenafter", expect_stderr=True)
-    # One of the effects of not cleaning up is broken scripts:
-    script_py = script.bin_path / "script.py"
-    assert script_py.exists(), script_py
-    with open(script_py) as f:
-        shebang = f.readline().strip()
-    assert shebang != "#!python", shebang
+    res = script.pip_install_local("wheelbrokenafter", expect_error=True)
+    assert "ERROR: Failed building wheel for wheelbrokenafter" in res.stderr
     # OK, assert that we *said* we were cleaning up:
     # /!\ if in need to change this, also change test_pep517_no_legacy_cleanup
     assert "Running setup.py clean for wheelbrokenafter" in str(res), str(res)
 
 
-@pytest.mark.usefixtures("with_wheel")
 def test_install_builds_wheels(script: PipTestEnvironment, data: TestData) -> None:
     # We need to use a subprocess to get the right value on Windows.
     res = script.run(
@@ -1577,39 +1704,31 @@ def test_install_builds_wheels(script: PipTestEnvironment, data: TestData) -> No
     # vcs coverage.
     to_install = data.packages.joinpath("requires_wheelbroken_upper")
     res = script.pip(
-        "install", "--no-index", "-f", data.find_links, to_install, expect_stderr=True
+        "install",
+        "--no-index",
+        "-f",
+        data.find_links,
+        to_install,
+        expect_error=True,  # error building wheelbroken
     )
-    expected = (
-        "Successfully installed requires-wheelbroken-upper-0"
-        " upper-2.0 wheelbroken-0.1"
-    )
-    # Must have installed it all
-    assert expected in str(res), str(res)
     wheels: List[str] = []
     for _, _, files in os.walk(wheels_cache):
         wheels.extend(f for f in files if f.endswith(".whl"))
-    # and built wheels for upper and wheelbroken
+    # Built wheel for upper
     assert "Building wheel for upper" in str(res), str(res)
+    # Built wheel for wheelbroken, but failed
     assert "Building wheel for wheelb" in str(res), str(res)
+    assert "Failed to build wheelbroken" in str(res), str(res)
     # Wheels are built for local directories, but not cached.
     assert "Building wheel for requir" in str(res), str(res)
-    # wheelbroken has to run install
     # into the cache
     assert wheels != [], str(res)
-    # and installed from the wheel
-    assert "Running setup.py install for upper" not in str(res), str(res)
-    # Wheels are built for local directories, but not cached.
-    assert "Running setup.py install for requir" not in str(res), str(res)
-    # wheelbroken has to run install
-    assert "Running setup.py install for wheelb" in str(res), str(res)
-    # We want to make sure pure python wheels do not have an implementation tag
     assert wheels == [
         "Upper-2.0-py{}-none-any.whl".format(sys.version_info[0]),
     ]
 
 
-@pytest.mark.usefixtures("with_wheel")
-def test_install_no_binary_disables_building_wheels(
+def test_install_no_binary_builds_wheels(
     script: PipTestEnvironment, data: TestData
 ) -> None:
     to_install = data.packages.joinpath("requires_wheelbroken_upper")
@@ -1620,29 +1739,17 @@ def test_install_no_binary_disables_building_wheels(
         "-f",
         data.find_links,
         to_install,
-        expect_stderr=True,
+        expect_error=True,  # error building wheelbroken
     )
-    expected = (
-        "Successfully installed requires-wheelbroken-upper-0"
-        " upper-2.0 wheelbroken-0.1"
-    )
-    # Must have installed it all
-    assert expected in str(res), str(res)
-    # and built wheels for wheelbroken only
+    # Wheels are built for all requirements
     assert "Building wheel for wheelb" in str(res), str(res)
-    # Wheels are built for local directories, but not cached across runs
     assert "Building wheel for requir" in str(res), str(res)
-    # Don't build wheel for upper which was blacklisted
-    assert "Building wheel for upper" not in str(res), str(res)
-    # Wheels are built for local directories, but not cached across runs
-    assert "Running setup.py install for requir" not in str(res), str(res)
-    # And these two fell back to sdist based installed.
-    assert "Running setup.py install for wheelb" in str(res), str(res)
-    assert "Running setup.py install for upper" in str(res), str(res)
+    assert "Building wheel for upper" in str(res), str(res)
+    # Wheelbroken failed to build
+    assert "Failed to build wheelbroken" in str(res), str(res)
 
 
 @pytest.mark.network
-@pytest.mark.usefixtures("with_wheel")
 def test_install_no_binary_builds_pep_517_wheel(
     script: PipTestEnvironment, data: TestData
 ) -> None:
@@ -1653,11 +1760,9 @@ def test_install_no_binary_builds_pep_517_wheel(
     assert expected in str(res), str(res)
 
     assert "Building wheel for pep517-setup" in str(res), str(res)
-    assert "Running setup.py install for pep517-set" not in str(res), str(res)
 
 
 @pytest.mark.network
-@pytest.mark.usefixtures("with_wheel")
 def test_install_no_binary_uses_local_backend(
     script: PipTestEnvironment, data: TestData, tmpdir: Path
 ) -> None:
@@ -1671,8 +1776,7 @@ def test_install_no_binary_uses_local_backend(
     assert os.path.isfile(marker), "Local PEP 517 backend not used"
 
 
-@pytest.mark.usefixtures("with_wheel")
-def test_install_no_binary_disables_cached_wheels(
+def test_install_no_binary_uses_cached_wheels(
     script: PipTestEnvironment, data: TestData
 ) -> None:
     # Seed the cache
@@ -1688,10 +1792,8 @@ def test_install_no_binary_disables_cached_wheels(
         expect_stderr=True,
     )
     assert "Successfully installed upper-2.0" in str(res), str(res)
-    # No wheel building for upper, which was blacklisted
+    # upper is built and not obtained from cache
     assert "Building wheel for upper" not in str(res), str(res)
-    # Must have used source, not a cached wheel to install upper.
-    assert "Running setup.py install for upper" in str(res), str(res)
 
 
 def test_install_editable_with_wrong_egg_name(
@@ -1812,7 +1914,6 @@ def test_install_incompatible_python_requires_editable(
     assert _get_expected_error_text() in result.stderr, str(result)
 
 
-@pytest.mark.usefixtures("with_wheel")
 def test_install_incompatible_python_requires_wheel(script: PipTestEnvironment) -> None:
     script.scratch_path.joinpath("pkga").mkdir()
     pkga_path = script.scratch_path / "pkga"
@@ -1942,28 +2043,6 @@ def test_installing_scripts_on_path_does_not_print_warning(
     assert "--no-warn-script-location" not in result.stderr
 
 
-def test_installed_files_recorded_in_deterministic_order(
-    script: PipTestEnvironment, data: TestData
-) -> None:
-    """
-    Ensure that we record the files installed by a package in a deterministic
-    order, to make installs reproducible.
-    """
-    to_install = data.packages.joinpath("FSPkg")
-    result = script.pip("install", to_install)
-    fspkg_folder = script.site_packages / "fspkg"
-    egg_info = f"FSPkg-0.1.dev0-py{pyversion}.egg-info"
-    installed_files_path = script.site_packages / egg_info / "installed-files.txt"
-    result.did_create(fspkg_folder)
-    result.did_create(installed_files_path)
-
-    installed_files_path = result.files_created[installed_files_path].full
-    installed_files_lines = [
-        p for p in Path(installed_files_path).read_text().split("\n") if p
-    ]
-    assert installed_files_lines == sorted(installed_files_lines)
-
-
 def test_install_conflict_results_in_warning(
     script: PipTestEnvironment, data: TestData
 ) -> None:
@@ -2044,7 +2123,7 @@ def test_target_install_ignores_distutils_config_install_prefix(
     result.did_not_create(relative_script_base)
 
 
-@pytest.mark.incompatible_with_test_venv
+@pytest.mark.usefixtures("enable_user_site")
 def test_user_config_accepted(script: PipTestEnvironment) -> None:
     # user set in the config file is parsed as 0/1 instead of True/False.
     # Check that this doesn't cause a problem.
@@ -2172,6 +2251,10 @@ def test_error_all_yanked_files_and_no_pin(
     ), str(result)
 
 
+@pytest.mark.skipif(
+    sys.platform == "linux" and sys.version_info < (3, 8),
+    reason="Custom SSL certification not running well in CI",
+)
 @pytest.mark.parametrize(
     "install_args",
     [
@@ -2254,7 +2337,6 @@ def test_install_skip_work_dir_pkg(script: PipTestEnvironment, data: TestData) -
 def test_install_verify_package_name_normalization(
     script: PipTestEnvironment, package_name: str
 ) -> None:
-
     """
     Test that install of a package again using a name which
     normalizes to the original package name, is a no-op
@@ -2286,3 +2368,103 @@ def test_install_dry_run(script: PipTestEnvironment, data: TestData) -> None:
     )
     assert "Would install simple-3.0" in result.stdout
     assert "Successfully installed" not in result.stdout
+
+
+@pytest.mark.skipif(
+    sys.version_info < (3, 11),
+    reason="3.11 required to find distributions via importlib metadata",
+)
+def test_install_existing_memory_distribution(script: PipTestEnvironment) -> None:
+    sitecustomize_text = textwrap.dedent(
+        """
+        import sys
+        from importlib.metadata import Distribution, DistributionFinder
+
+
+        EXAMPLE_METADATA = '''Metadata-Version: 2.1
+        Name: example
+        Version: 1.0.0
+
+        '''
+
+        class ExampleDistribution(Distribution):
+            def locate_file(self, path):
+                return path
+
+            def read_text(self, filename):
+                if filename == 'METADATA':
+                    return EXAMPLE_METADATA
+
+
+        class CustomFinder(DistributionFinder):
+            def find_distributions(self, context=None):
+                return [ExampleDistribution()]
+
+
+        sys.meta_path.append(CustomFinder())
+        """
+    )
+    with open(script.site_packages_path / "sitecustomize.py", "w") as sitecustomize:
+        sitecustomize.write(sitecustomize_text)
+
+    result = script.pip("install", "example")
+
+    assert "Requirement already satisfied: example in <memory>" in result.stdout
+
+
+def test_install_pip_prints_req_chain_local(script: PipTestEnvironment) -> None:
+    """
+    Test installing a local package with a dependency and check that the
+    dependency chain is reported.
+    """
+
+    req_path = script.scratch_path.joinpath("requirements.txt")
+    req_path.write_text("base==0.1.0")
+
+    create_basic_wheel_for_package(
+        script,
+        "base",
+        "0.1.0",
+        depends=["dep"],
+    )
+    dep_path = create_basic_wheel_for_package(
+        script,
+        "dep",
+        "0.1.0",
+    )
+
+    result = script.pip(
+        "install",
+        "--no-cache-dir",
+        "--no-index",
+        "--find-links",
+        script.scratch_path,
+        "-r",
+        req_path,
+    )
+    assert_re_match(
+        rf"Processing .*{re.escape(os.path.basename(dep_path))} "
+        rf"\(from base==0.1.0->-r {re.escape(str(req_path))} \(line 1\)\)",
+        result.stdout,
+    )
+
+
+@pytest.mark.network
+def test_install_pip_prints_req_chain_pypi(script: PipTestEnvironment) -> None:
+    """
+    Test installing a package with a dependency from PyPI and check that the
+    dependency chain is reported.
+    """
+    req_path = script.scratch_path.joinpath("requirements.txt")
+    req_path.write_text("Paste[openid]==1.7.5.1")
+
+    result = script.pip(
+        "install",
+        "-r",
+        req_path,
+    )
+
+    assert (
+        f"Collecting python-openid "
+        f"(from Paste[openid]==1.7.5.1->-r {req_path} (line 1))" in result.stdout
+    )
